use clap::{Parser, ValueEnum};
use psxcdgen_ex::{encoder::{EncodingFunctions, psx::{encode_psx_image, calculate_psx_lbas}}, file_writer::{ImageType, write_image}, config_reader};
use std::{path::PathBuf, };
use tool_helper::Error;

#[derive(Parser)]
#[clap(about = "Creates an ISO image from a description file", long_about = None)]
struct CommandLine {
    #[clap(value_enum, value_parser)]
    system_type: SystemType,

    #[clap(value_enum, value_parser)]
    output_type: ImageType,

    #[clap(short='o')]
    output_file: PathBuf,

    #[clap(value_parser)]
    input_file: PathBuf,
}

#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum)]
enum SystemType {
    Psx
}

impl SystemType {
    pub fn get_encoding_functions(self) -> EncodingFunctions {
        match self {
            SystemType::Psx => EncodingFunctions{lba_calculator: calculate_psx_lbas, encoder: encode_psx_image}
        }
    }
}

fn run_main(cmd_line: CommandLine) -> Result<(), Error> {
    let encoding_functions = cmd_line.system_type.get_encoding_functions();
    let desc               = psxcdgen_ex::process(config_reader::parse_xml(std::fs::read_to_string(cmd_line.input_file)?)?, encoding_functions.lba_calculator)?;
    
    println!("\n<== Planschbecken ==>");
    for rand_item in desc.create_file_map() {
        println!("{}", rand_item.0);
    }
    println!("\n<== Planschbecken ==>");
    
    psxcdgen_ex::dump_content(&desc, tool_helper::open_output(None)?)?;
    println!("\n<== Planschbecken ==>\nStart encoding");
    write_image(desc, encoding_functions.encoder, cmd_line.output_type, cmd_line.output_file)
}

fn main() {
    match CommandLine::try_parse() {
        Ok(cmd_line) => {
            match run_main(cmd_line) {
                Ok(_)      => println!("\n<== Planschbecken End ==>"),
                Err(error) => println!("{}", error)
            }
        },
        Err(error) => {
            println!("{}", error);
        }
    }
}